"""Settings Dialog Class"""

import logging
from PyQt6 import QtWidgets, uic

try:
    import sounddevice as sd
except OSError:
    sd = None


class Settings(QtWidgets.QDialog):
    """Settings dialog"""

    def __init__(self, app_data_path, pref, parent=None):
        """initialize dialog"""
        super().__init__(parent)
        self.logger = logging.getLogger("settings")
        uic.loadUi(app_data_path / "configuration.ui", self)
        # self.tabWidget.setTabVisible(5, False)
        # self.group_tab.hide()
        self.buttonBox.accepted.connect(self.save_changes)
        self.usecwdaemon_radioButton.clicked.connect(self.set_cwdaemon_port_hint)
        self.usepywinkeyer_radioButton.clicked.connect(self.set_winkeyer_port_hint)
        self.usecwviacat_radioButton.clicked.connect(self.set_catforcw_port_hint)
        self.cwport_field.setToolTip(
            "Usually 6789 for cwdaemon and 8000 for pywinkeyer."
        )
        self.rigcontrolport_field.setToolTip(
            "Usually 4532 for rigctld and 12345 for flrig."
        )
        self.preference = pref
        if sd:
            self.devices = sd.query_devices()
        else:
            self.devices = []
        self.setup()

    def setup(self):
        """setup dialog"""

        self.send_rtc_scores.setChecked(
            bool(self.preference.get("send_rtc_scores", False))
        )

        value = self.preference.get("rtc_url", "")
        index = self.rtc_url.findText(value)
        if index != -1:
            self.rtc_url.setCurrentIndex(index)

        self.rtc_user.setText(str(self.preference.get("rtc_user", "")))
        self.rtc_pass.setText(str(self.preference.get("rtc_pass", "")))
        self.rtc_interval.setText(str(self.preference.get("rtc_interval", "2")))

        self.auto_cq_delay.setText(str(self.preference.get("auto_cq_interval", "15")))

        self.connect_to_server.setChecked(bool(self.preference.get("useserver", False)))

        self.use_call_history.setChecked(
            bool(self.preference.get("use_call_history", False))
        )

        self.qsy_on_change.setChecked(bool(self.preference.get("sandpqsy", False)))

        self.use_esm.setChecked(bool(self.preference.get("use_esm", False)))

        value = self.preference.get("esm_agn", "DISABLED")
        index = self.esm_agn.findText(value)
        if index != -1:
            self.esm_agn.setCurrentIndex(index)

        value = self.preference.get("esm_cq", "DISABLED")
        index = self.esm_cq.findText(value)
        if index != -1:
            self.esm_cq.setCurrentIndex(index)

        value = self.preference.get("esm_exch", "DISABLED")
        index = self.esm_exch.findText(value)
        if index != -1:
            self.esm_exch.setCurrentIndex(index)

        value = self.preference.get("esm_hiscall", "DISABLED")
        index = self.esm_hiscall.findText(value)
        if index != -1:
            self.esm_hiscall.setCurrentIndex(index)

        value = self.preference.get("esm_mycall", "DISABLED")
        index = self.esm_mycall.findText(value)
        if index != -1:
            self.esm_mycall.setCurrentIndex(index)

        value = self.preference.get("esm_qrz", "DISABLED")
        index = self.esm_qrz.findText(value)
        if index != -1:
            self.esm_qrz.setCurrentIndex(index)

        value = self.preference.get("esm_qsob4", "DISABLED")
        index = self.esm_qsob4.findText(value)
        if index != -1:
            self.esm_qsob4.setCurrentIndex(index)

        for device in self.devices:
            if device.get("max_output_channels"):
                self.sounddevice.addItem(device.get("name"))
        value = self.preference.get("sounddevice", "default")
        index = self.sounddevice.findText(value)
        if index != -1:
            self.sounddevice.setCurrentIndex(index)
        self.useqrz_radioButton.setChecked(bool(self.preference.get("useqrz")))
        self.usehamqth_radioButton.setChecked(bool(self.preference.get("usehamqth")))
        self.lookup_user_name_field.setText(
            str(self.preference.get("lookupusername", ""))
        )
        self.lookup_password_field.setText(
            str(self.preference.get("lookuppassword", ""))
        )
        self.rigcontrolip_field.setText(str(self.preference.get("CAT_ip", "")))
        self.rigcontrolport_field.setText(str(self.preference.get("CAT_port", "")))
        self.catpoll_field.setText(str(self.preference.get("CAT_polldelta", 500)))
        self.userigctld_radioButton.setChecked(bool(self.preference.get("userigctld")))
        self.useflrig_radioButton.setChecked(bool(self.preference.get("useflrig")))

        self.rotctld_address.setText(str(self.preference.get("rotctld_address", "")))
        self.rotctld_port.setText(str(self.preference.get("rotctld_port", "")))

        self.cwip_field.setText(str(self.preference.get("cwip", "")))
        if self.preference.get("cwport", ""):
            self.cwport_field.setText(str(self.preference.get("cwport", "")))
        else:
            self.cwport_field.setText("")
        self.usecwdaemon_radioButton.setChecked(
            bool(self.preference.get("cwtype") == 1)
        )
        self.usepywinkeyer_radioButton.setChecked(
            bool(self.preference.get("cwtype") == 2)
        )
        self.usecwviacat_radioButton.setChecked(
            bool(self.preference.get("cwtype") == 3)
        )
        if self.preference.get("cwtype") == 1:
            self.set_cwdaemon_port_hint()
        elif self.preference.get("cwtype") == 2:
            self.set_winkeyer_port_hint()
        elif self.preference.get("cwtype") == 3:
            self.set_catforcw_port_hint()
        self.cwpaddingchar_field.setText(self.preference.get("cwpaddingchar", "T"))
        self.cwpaddinglength_field.setText(
            str(self.preference.get("cwpaddinglength", "3"))
        )

        self.connect_to_server.setChecked(bool(self.preference.get("useserver", False)))
        self.multicast_group.setText(str(self.preference.get("multicast_group", "")))
        self.multicast_port.setText(str(self.preference.get("multicast_port", "")))
        self.interface_ip.setText(str(self.preference.get("interface_ip", "")))

        self.send_n1mm_packets.setChecked(
            bool(self.preference.get("send_n1mm_packets"))
        )
        self.n1mm_station_name.setText(
            str(self.preference.get("n1mm_station_name", ""))
        )
        self.n1mm_operator.setText(str(self.preference.get("n1mm_operator", "")))
        self.n1mm_radioport.setText(str(self.preference.get("n1mm_radioport", "")))
        self.n1mm_contactport.setText(str(self.preference.get("n1mm_contactport", "")))
        self.n1mm_lookupport.setText(str(self.preference.get("n1mm_lookupport", "")))
        self.n1mm_scoreport.setText(str(self.preference.get("n1mm_scoreport", "")))
        self.send_n1mm_radio.setChecked(bool(self.preference.get("send_n1mm_radio")))
        self.send_n1mm_contact.setChecked(
            bool(self.preference.get("send_n1mm_contact"))
        )
        self.send_n1mm_lookup.setChecked(bool(self.preference.get("send_n1mm_lookup")))
        self.send_n1mm_score.setChecked(bool(self.preference.get("send_n1mm_score")))

        self.cluster_server_field.setText(
            str(self.preference.get("cluster_server", "dxc.nc7j.com"))
        )
        self.cluster_port_field.setText(str(self.preference.get("cluster_port", 7373)))
        self.cluster_password_field.setText(
            str(self.preference.get("cluster_password", ""))
        )
        self.cluster_filter.setText(self.preference.get("cluster_filter", ""))
        value = self.preference.get("cluster_mode", "")
        index = self.cluster_mode.findText(value)
        if index != -1:
            self.cluster_mode.setCurrentIndex(index)

        self.activate_160m.setChecked(bool("160" in self.preference.get("bands", [])))
        self.activate_80m.setChecked(bool("80" in self.preference.get("bands", [])))
        self.activate_60m.setChecked(bool("60" in self.preference.get("bands", [])))
        self.activate_40m.setChecked(bool("40" in self.preference.get("bands", [])))
        self.activate_30m.setChecked(bool("30" in self.preference.get("bands", [])))
        self.activate_20m.setChecked(bool("20" in self.preference.get("bands", [])))
        self.activate_17m.setChecked(bool("17" in self.preference.get("bands", [])))
        self.activate_15m.setChecked(bool("15" in self.preference.get("bands", [])))
        self.activate_12m.setChecked(bool("12" in self.preference.get("bands", [])))
        self.activate_10m.setChecked(bool("10" in self.preference.get("bands", [])))
        self.activate_6m.setChecked(bool("6" in self.preference.get("bands", [])))
        self.activate_4m.setChecked(bool("4" in self.preference.get("bands", [])))
        self.activate_2m.setChecked(bool("2" in self.preference.get("bands", [])))
        self.activate_1dot25.setChecked(
            bool("1.25" in self.preference.get("bands", []))
        )
        self.activate_70cm.setChecked(bool("70cm" in self.preference.get("bands", [])))
        self.activate_33cm.setChecked(bool("33cm" in self.preference.get("bands", [])))
        self.activate_23cm.setChecked(bool("23cm" in self.preference.get("bands", [])))

    def set_cwdaemon_port_hint(self):
        """Sets placeholder hint for the CW interface."""
        self.cwip_field.setPlaceholderText("127.0.0.1")
        self.cwport_field.setPlaceholderText("6789")

    def set_winkeyer_port_hint(self):
        """Sets placeholder hint for the CW interface."""
        self.cwip_field.setPlaceholderText("127.0.0.1")
        self.cwport_field.setPlaceholderText("8000")

    def set_catforcw_port_hint(self):
        """Sets placeholder hint for the CW interface."""
        self.cwip_field.setPlaceholderText("")
        self.cwport_field.setPlaceholderText("")

    def save_changes(self):
        """
        Write preferences to json file.
        """
        self.preference["useserver"] = self.connect_to_server.isChecked()
        self.preference["send_rtc_scores"] = self.send_rtc_scores.isChecked()
        self.preference["rtc_url"] = self.rtc_url.currentText()
        self.preference["rtc_user"] = self.rtc_user.text()
        self.preference["rtc_pass"] = self.rtc_pass.text()
        try:
            self.preference["rtc_interval"] = int(self.rtc_interval.text())
        except ValueError:
            self.preference["rtc_interval"] = 2

        try:
            self.preference["auto_cq_interval"] = int(self.auto_cq_delay.text())
        except ValueError:
            self.preference["auto_cq_interval"] = 15

        self.auto_cq_delay.setText(str(self.preference.get("auto_cq_interval", "15")))

        self.preference["use_call_history"] = self.use_call_history.isChecked()

        self.preference["sandpqsy"] = self.qsy_on_change.isChecked()

        self.preference["use_esm"] = self.use_esm.isChecked()
        self.preference["esm_cq"] = self.esm_cq.currentText()
        self.preference["esm_agn"] = self.esm_agn.currentText()
        self.preference["esm_exch"] = self.esm_exch.currentText()
        self.preference["esm_hiscall"] = self.esm_hiscall.currentText()
        self.preference["esm_mycall"] = self.esm_mycall.currentText()
        self.preference["esm_qrz"] = self.esm_qrz.currentText()
        self.preference["esm_qsob4"] = self.esm_qsob4.currentText()

        self.preference["sounddevice"] = self.sounddevice.currentText()
        self.preference["useqrz"] = self.useqrz_radioButton.isChecked()
        # self.preference["usehamdb"] = self.usehamdb_radioButton.isChecked()
        self.preference["usehamqth"] = self.usehamqth_radioButton.isChecked()
        self.preference["lookupusername"] = self.lookup_user_name_field.text()
        self.preference["lookuppassword"] = self.lookup_password_field.text()
        self.preference["CAT_ip"] = self.rigcontrolip_field.text()
        try:
            self.preference["CAT_port"] = int(self.rigcontrolport_field.text())
        except ValueError:
            ...
        try:
            self.preference["CAT_polldelta"] = int(self.catpoll_field.text())
        except ValueError:
            ...
        self.preference["userigctld"] = self.userigctld_radioButton.isChecked()
        self.preference["useflrig"] = self.useflrig_radioButton.isChecked()

        self.preference["rotctld_address"] = self.rotctld_address.text()
        try:
            self.preference["rotctld_port"] = int(self.rotctld_port.text())
        except ValueError:
            self.preference["rotctld_port"] = 4533

        self.preference["cwip"] = self.cwip_field.text()
        try:
            self.preference["cwport"] = int(self.cwport_field.text())
        except ValueError:
            self.preference["cwport"] = None
            ...
        self.preference["cwpaddingchar"] = self.cwpaddingchar_field.text()
        try:
            self.preference["cwpaddinglength"] = int(self.cwpaddinglength_field.text())
        except ValueError:
            self.preference["cwpaddinglength"] = 3
        self.preference["cwtype"] = 0
        if self.usecwdaemon_radioButton.isChecked():
            self.preference["cwtype"] = 1
        elif self.usepywinkeyer_radioButton.isChecked():
            self.preference["cwtype"] = 2
        elif self.usecwviacat_radioButton.isChecked():
            self.preference["cwtype"] = 3
        self.preference["useserver"] = self.connect_to_server.isChecked()
        self.preference["multicast_group"] = self.multicast_group.text()
        self.preference["multicast_port"] = self.multicast_port.text()
        self.preference["interface_ip"] = self.interface_ip.text()

        self.preference["send_n1mm_packets"] = self.send_n1mm_packets.isChecked()

        self.preference["send_n1mm_radio"] = self.send_n1mm_radio.isChecked()
        self.preference["send_n1mm_contact"] = self.send_n1mm_contact.isChecked()
        self.preference["send_n1mm_lookup"] = self.send_n1mm_lookup.isChecked()
        self.preference["send_n1mm_score"] = self.send_n1mm_score.isChecked()

        self.preference["n1mm_station_name"] = self.n1mm_station_name.text()
        self.preference["n1mm_operator"] = self.n1mm_operator.text()
        # self.preference["n1mm_ip"] = self.n1mm_ip.text()
        self.preference["n1mm_radioport"] = self.n1mm_radioport.text()
        self.preference["n1mm_contactport"] = self.n1mm_contactport.text()
        self.preference["n1mm_lookupport"] = self.n1mm_lookupport.text()
        self.preference["n1mm_scoreport"] = self.n1mm_scoreport.text()
        self.preference["cluster_server"] = self.cluster_server_field.text()
        try:
            self.preference["cluster_port"] = int(self.cluster_port_field.text())
        except:
            ...
        self.preference["cluster_password"] = self.cluster_password_field.text()
        self.preference["cluster_filter"] = self.cluster_filter.text()
        self.preference["cluster_mode"] = self.cluster_mode.currentText()
        bandlist = list()
        if self.activate_160m.isChecked():
            bandlist.append("160")
        if self.activate_80m.isChecked():
            bandlist.append("80")
        if self.activate_60m.isChecked():
            bandlist.append("60")
        if self.activate_40m.isChecked():
            bandlist.append("40")
        if self.activate_30m.isChecked():
            bandlist.append("30")
        if self.activate_20m.isChecked():
            bandlist.append("20")
        if self.activate_17m.isChecked():
            bandlist.append("17")
        if self.activate_15m.isChecked():
            bandlist.append("15")
        if self.activate_12m.isChecked():
            bandlist.append("12")
        if self.activate_10m.isChecked():
            bandlist.append("10")
        if self.activate_6m.isChecked():
            bandlist.append("6")
        if self.activate_4m.isChecked():
            bandlist.append("4")
        if self.activate_2m.isChecked():
            bandlist.append("2")
        if self.activate_1dot25.isChecked():
            bandlist.append("1.25")
        if self.activate_70cm.isChecked():
            bandlist.append("70cm")
        if self.activate_33cm.isChecked():
            bandlist.append("33cm")
        if self.activate_23cm.isChecked():
            bandlist.append("23cm")
        self.preference["bands"] = bandlist



"""General Logging plugin"""

# pylint: disable=invalid-name, unused-argument, unused-variable, c-extension-no-member, unused-import

import logging

from PyQt6 import QtWidgets

from not1mm.lib.ham_utility import get_logged_band
from not1mm.lib.plugin_common import gen_adif, get_points
from not1mm.lib.version import __version__

logger = logging.getLogger(__name__)
EXCHANGE_HINT = ""
name = "General Logging"
cabrillo_name = "General-Logging"
mode = "BOTH"  # CW SSB BOTH RTTY
columns = [0, 1, 2, 3, 4, 16, 17]
columns = [
    "YYYY-MM-DD HH:MM:SS",
    "Call",
    "Freq",
    "Mode",
    "Snt",
    "Rcv",
    "Name",
    "Comment",
]

# 1 once per contest, 2 work each band, 3 each band/mode, 4 no dupe checking
dupe_type = 4


def init_contest(self):
    """setup plugin"""
    set_tab_next(self)
    set_tab_prev(self)
    interface(self)
    self.next_field = self.other_1


def interface(self):
    """Setup user interface"""
    self.field1.show()
    self.field2.show()
    self.field3.show()
    self.field4.show()
    self.snt_label.setText("SNT")
    self.field1.setAccessibleName("RST Sent")
    self.other_label.setText("Name")
    self.field3.setAccessibleName("Name")
    self.exch_label.setText("Comment")
    self.field4.setAccessibleName("Comment")


def reset_label(self):
    """reset label after field cleared"""


def set_tab_next(self):
    """Set TAB Advances"""
    self.tab_next = {
        self.callsign: self.sent,
        self.sent: self.receive,
        self.receive: self.other_1,
        self.other_1: self.other_2,
        self.other_2: self.callsign,
    }


def set_tab_prev(self):
    """Set TAB Advances"""
    self.tab_prev = {
        self.callsign: self.other_2,
        self.sent: self.callsign,
        self.receive: self.sent,
        self.other_1: self.receive,
        self.other_2: self.other_1,
    }


def set_contact_vars(self):
    """Contest Specific"""
    self.contact["SNT"] = self.sent.text()
    self.contact["RCV"] = self.receive.text()
    self.contact["Name"] = self.other_1.text()
    self.contact["Comment"] = self.other_2.text()


def predupe(self):
    """called after callsign entered"""


def prefill(self):
    """Fill SentNR"""


def points(self):
    """Calc point"""


def show_mults(self):
    """Return display string for mults"""


def show_qso(self):
    """Return qso count"""
    result = self.database.fetch_qso_count()
    if result:
        return int(result.get("qsos", 0))
    return 0


def calc_score(self):
    """Return calculated score"""
    result = self.database.fetch_points()


def adif(self):
    """Call the generate ADIF function"""
    gen_adif(self, cabrillo_name)


def cabrillo(self, file_encoding):
    """Generates Cabrillo file. Maybe."""


def recalculate_mults(self):
    """Recalculates multipliers after change in logged qso."""


def process_esm(self, new_focused_widget=None, with_enter=False):
    """ESM State Machine"""

    # self.pref["run_state"]

    # -----===== Assigned F-Keys =====-----
    # self.esm_dict["CQ"]
    # self.esm_dict["EXCH"]
    # self.esm_dict["QRZ"]
    # self.esm_dict["AGN"]
    # self.esm_dict["HISCALL"]
    # self.esm_dict["MYCALL"]
    # self.esm_dict["QSOB4"]

    # ----==== text fields ====----
    # self.callsign
    # self.sent
    # self.receive
    # self.other_1
    # self.other_2

    if new_focused_widget is not None:
        self.current_widget = self.inputs_dict.get(new_focused_widget)

    # print(f"checking esm {self.current_widget=} {with_enter=} {self.pref.get("run_state")=}")

    for a_button in [
        self.F1,
        self.F2,
        self.F3,
        self.F4,
        self.F5,
        self.F6,
        self.F7,
        self.F8,
        self.F9,
        self.F10,
        self.F11,
        self.F12,
    ]:
        self.restore_button_color(a_button)

    buttons_to_send = []

    if self.pref.get("run_state"):
        if self.current_widget == "callsign":
            if len(self.callsign.text()) < 3:
                self.make_button_green(self.esm_dict["CQ"])
                buttons_to_send.append(self.esm_dict["CQ"])
            elif len(self.callsign.text()) > 2:
                self.make_button_green(self.esm_dict["HISCALL"])
                self.make_button_green(self.esm_dict["EXCH"])
                buttons_to_send.append(self.esm_dict["HISCALL"])
                buttons_to_send.append(self.esm_dict["EXCH"])

        elif self.current_widget in ["other_1", "other_2"]:
            self.make_button_green(self.esm_dict["QRZ"])
            buttons_to_send.append(self.esm_dict["QRZ"])
            buttons_to_send.append("LOGIT")

        if with_enter is True and bool(len(buttons_to_send)):
            for button in buttons_to_send:
                if button:
                    if button == "LOGIT":
                        self.save_contact()
                        continue
                    self.process_function_key(button)
    else:
        if self.current_widget == "callsign":
            if len(self.callsign.text()) > 2:
                self.make_button_green(self.esm_dict["MYCALL"])
                buttons_to_send.append(self.esm_dict["MYCALL"])

        elif self.current_widget in ["other_1", "other_2"]:
            self.make_button_green(self.esm_dict["QRZ"])
            buttons_to_send.append(self.esm_dict["QRZ"])
            buttons_to_send.append("LOGIT")

        if with_enter is True and bool(len(buttons_to_send)):
            for button in buttons_to_send:
                if button:
                    if button == "LOGIT":
                        self.save_contact()
                        continue
                    self.process_function_key(button)


def populate_history_info_line(self):
    result = self.database.fetch_call_history(self.callsign.text())
    if result:
        self.history_info.setText(
            f"{result.get('Call', '')}, {result.get('Name', '')}, {result.get('UserText','...')}"
        )
    else:
        self.history_info.setText("")


def check_call_history(self):
    """"""
    result = self.database.fetch_call_history(self.callsign.text())
    if result:
        self.history_info.setText(f"{result.get('UserText','')}")
        if self.other_1.text() == "":
            self.other_1.setText(f"{result.get('Name', '')}")


def set_self(the_outie):
    """..."""
    globals()["ALTEREGO"] = the_outie


def ft8_handler(the_packet: dict):
    """Process FT8 QSO packets
    FT8
    {
        'CALL': 'KE0OG',
        'GRIDSQUARE': 'DM10AT',
        'MODE': 'FT8',
        'RST_SENT': '',
        'RST_RCVD': '',
        'QSO_DATE': '20210329',
        'TIME_ON': '183213',
        'QSO_DATE_OFF': '20210329',
        'TIME_OFF': '183213',
        'BAND': '20M',
        'FREQ': '14.074754',
        'STATION_CALLSIGN': 'K6GTE',
        'MY_GRIDSQUARE': 'DM13AT',
        'CONTEST_ID': 'ARRL-FIELD-DAY',
        'SRX_STRING': '1D UT',
        'CLASS': '1D',
        'ARRL_SECT': 'UT'
    }
    FlDigi
    {
        'FREQ': '7.029500',
        'CALL': 'DL2DSL',
        'MODE': 'RTTY',
        'NAME': 'BOB',
        'QSO_DATE': '20240904',
        'QSO_DATE_OFF': '20240904',
        'TIME_OFF': '212825',
        'TIME_ON': '212800',
        'RST_RCVD': '599',
        'RST_SENT': '599',
        'BAND': '40M',
        'COUNTRY': 'FED. REP. OF GERMANY',
        'CQZ': '14',
        'STX': '000',
        'STX_STRING': '1D ORG',
        'CLASS': '1D',
        'ARRL_SECT': 'DX',
        'TX_PWR': '0',
        'OPERATOR': 'K6GTE',
        'STATION_CALLSIGN': 'K6GTE',
        'MY_GRIDSQUARE': 'DM13AT',
        'MY_CITY': 'ANAHEIM, CA',
        'MY_STATE': 'CA'
    }

    """
    # print(f"\n{the_packet=}\n")
    if ALTEREGO is not None:
        ALTEREGO.callsign.setText(the_packet.get("CALL"))
        ALTEREGO.contact["Call"] = the_packet.get("CALL", "")
        ALTEREGO.contact["SNT"] = the_packet.get("RST_SENT", "59")
        ALTEREGO.contact["RCV"] = the_packet.get("RST_RCVD", "59")
        my_grid = the_packet.get("MY_GRIDSQUARE", "")
        if my_grid:
            if len(my_grid) > 4:
                my_grid = my_grid[:4]
        their_grid = the_packet.get("GRIDSQUARE", "")
        if their_grid:
            ALTEREGO.contact["GridSquare"] = their_grid
        if the_packet.get("SUBMODE"):
            ALTEREGO.contact["Mode"] = the_packet.get("SUBMODE", "ERR")
        else:
            ALTEREGO.contact["Mode"] = the_packet.get("MODE", "ERR")
        ALTEREGO.contact["Freq"] = round(float(the_packet.get("FREQ", "0.0")) * 1000, 2)
        ALTEREGO.contact["QSXFreq"] = round(
            float(the_packet.get("FREQ", "0.0")) * 1000, 2
        )
        ALTEREGO.contact["Band"] = get_logged_band(
            str(int(float(the_packet.get("FREQ", "0.0")) * 1000000))
        )
        ALTEREGO.save_contact()



#!/usr/bin/env python3
"""
not1mm Contest logger
Email: michael.bridak@gmail.com
GPL V3
Class: CheckWindow
Purpose: Onscreen widget to show possible matches to callsigns entered in the main window.
"""
# pylint: disable=no-name-in-module, unused-import, no-member, invalid-name, c-extension-no-member
# pylint: disable=logging-fstring-interpolation, line-too-long

from dataclasses import dataclass
import logging
import os
import queue
from typing import Optional
from json import loads
import Levenshtein

from PyQt6 import QtGui, uic
from PyQt6.QtWidgets import QLabel, QVBoxLayout, QWidget, QDockWidget, QApplication
from PyQt6.QtGui import QMouseEvent
from PyQt6.QtCore import pyqtSignal

import not1mm.fsutils as fsutils
from not1mm.lib.database import DataBase

from not1mm.lib.super_check_partial import SCP

logger = logging.getLogger(__name__)


class CheckWindow(QDockWidget):
    """The check window. Shows list or probable stations."""

    @dataclass
    class BackgroundColors:
        """Background color data for the checkpartial call signs."""

        character_remove_color: str
        character_add_color: str
        character_match_color: str

    message = pyqtSignal(dict)
    dbname = None
    pref = {}
    call = None
    masterLayout: QVBoxLayout = None
    dxcLayout: QVBoxLayout = None
    qsoLayout: QVBoxLayout = None
    background_colors_cache: Optional[BackgroundColors] = None

    masterScrollWidget: QWidget = None

    def __init__(self):
        super().__init__()
        self.active = False
        self.load_pref()
        self.dbname = fsutils.USER_DATA_PATH / self.pref.get(
            "current_database", "ham.db"
        )
        self.database = DataBase(self.dbname, fsutils.APP_DATA_PATH)
        self.database.current_contest = self.pref.get("contest", 0)

        uic.loadUi(fsutils.APP_DATA_PATH / "checkwindow.ui", self)
        self.mscp = SCP(fsutils.APP_DATA_PATH)
        self._udpwatch = None
        self.udp_fifo = queue.Queue()

        def invalidate_background_colors_cache_on_mode_change():
            self.background_colors_cache = None

        QApplication.instance().styleHints().colorSchemeChanged.connect(
            invalidate_background_colors_cache_on_mode_change
        )

    def msg_from_main(self, packet):
        """"""

        if packet.get("cmd", "") == "UPDATELOG":
            self.clear_lists()
            return

        if self.active is False:
            return
        if packet.get("cmd", "") == "CALLCHANGED":
            call = packet.get("call", "")
            self.call = call
            self.master_list(call)
            self.log_list(call)
            return
        if packet.get("cmd", "") == "CHECKSPOTS":
            self.populate_layout(self.dxcLayout, [])
            spots = packet.get("spots", [])
            self.telnet_list(spots)
            return
        if packet.get("cmd", "") == "NEWDB":
            ...
            # self.load_new_db()

    def setActive(self, mode: bool):
        self.active = bool(mode)

    def item_clicked(self, item):
        """docstring for item_clicked"""
        if item:
            cmd = {}
            cmd["cmd"] = "CHANGECALL"
            cmd["call"] = item
            self.message.emit(cmd)

    def load_pref(self) -> None:
        """
        Load preference file to get current db filename and sets the initial darkmode state.

        Parameters
        ----------
        None

        Returns
        -------
        None
        """
        try:
            if os.path.exists(fsutils.CONFIG_FILE):
                with open(
                    fsutils.CONFIG_FILE, "rt", encoding="utf-8"
                ) as file_descriptor:
                    self.pref = loads(file_descriptor.read())
                    logger.info(f"loaded config file from {fsutils.CONFIG_FILE}")
            else:
                self.pref["current_database"] = "ham.db"

        except IOError as exception:
            logger.critical("Error: %s", exception)

    def clear_lists(self) -> None:
        """
        Clear match lists.

        Parameters
        ----------
        None

        Returns
        -------
        None
        """
        self.populate_layout(self.masterLayout, [])
        self.populate_layout(self.qsoLayout, [])
        self.populate_layout(self.dxcLayout, [])

    def master_list(self, call: str) -> None:
        """
        Get MASTER.SCP matches to call and display in list.

        Parameters
        ----------
        call : str
        Call to get matches for

        Returns
        -------
        None
        """
        self.populate_layout(self.masterLayout, [])
        self.populate_layout(self.masterLayout, self.mscp.super_check(call))

    def log_list(self, call: str) -> None:
        """
        Get log matches to call and display in list.

        Parameters
        ----------
        call : str
        Call to get matches for

        Returns
        -------
        None
        """
        self.populate_layout(self.qsoLayout, [])
        if call:
            result = self.database.get_like_calls_and_bands(call)
            self.populate_layout(self.qsoLayout, result)

    def telnet_list(self, spots: list) -> None:
        """
        Get telnet matches to call and display in list.

        Parameters
        ----------
        spots : list
        List of spots to get matches for

        Returns
        -------
        None
        """
        self.populate_layout(self.dxcLayout, [])
        if spots:
            self.populate_layout(
                self.dxcLayout,
                filter(lambda x: x, [x.get("callsign", None) for x in spots]),
            )

    def populate_layout(self, layout, call_list) -> None:
        """Apply blackmagic to a layout."""
        for i in reversed(range(layout.count())):
            if layout.itemAt(i).widget():
                layout.itemAt(i).widget().setParent(None)
            else:
                layout.removeItem(layout.itemAt(i))
        background_colors = self.background_colors_for_mode()
        call_items = []
        for call in call_list:
            if call:
                if self.call:
                    label_text = ""
                    diff_score = 0
                    for tag, i1, i2, j1, j2 in Levenshtein.opcodes(call, self.call):
                        if tag == "equal":
                            label_text += call[i1:i2]
                            continue
                        elif tag == "replace":
                            label_text += f"<span style='background-color: {background_colors.character_remove_color};'>{call[i1:i2]}</span>"
                            diff_score += max((i2 - i1), (j2 - j1)) * (
                                len(call) + 1 - i2
                            )
                        elif tag == "insert" or tag == "delete":
                            label_text += f"<span style='background-color: {background_colors.character_add_color};'>{call[i1:i2]}</span>"
                            diff_score += max((i2 - i1), (j2 - j1)) * (len(call) - i2)
                    if call == self.call:
                        label_text = f"<span style='background-color: {background_colors.character_match_color};'>{call}</span>"
                    call_items.append((diff_score, label_text, call))

        call_items = sorted(call_items, key=lambda x: x[0])
        for i in reversed(range(layout.count())):
            if layout.itemAt(i).widget():
                layout.itemAt(i).widget().setParent(None)
            else:
                layout.removeItem(layout.itemAt(i))

        for _, label_text, call in call_items:
            label = CallLabel(label_text, call=call, callback=self.item_clicked)
            layout.addWidget(label)
        layout.addStretch(0)

    def background_colors_for_mode(self) -> "CheckWindow.BackgroundColors":
        """Returns appropriate background colors depending on dark or light mode.

        These are used for the checkpartial call signs.
        """
        if self.background_colors_cache is None:
            palette = self.widget.palette()
            text_lightness = palette.windowText().color().lightness()
            background_lightness = palette.window().color().lightness()
            if background_lightness < text_lightness:
                # dark mode
                self.background_colors_cache = CheckWindow.BackgroundColors(
                    character_remove_color="#dd3333",
                    character_add_color="#3333dd",
                    character_match_color="#33bb33",
                )
            else:
                # light mode
                self.background_colors_cache = CheckWindow.BackgroundColors(
                    character_remove_color="#ffcccc",
                    character_add_color="#ccccff",
                    character_match_color="#ccffcc",
                )
        return self.background_colors_cache


class CallLabel(QLabel):
    call: str = None

    def __init__(
        self,
        *args,
        call=None,
        callback=None,
    ):
        super().__init__(*args)
        self.call = call
        self.callback = callback

    def mouseReleaseEvent(self, e: QMouseEvent) -> None:
        if self.call and self.callback:
            self.callback(self.call)
